@@ -21,16 +21,55 @@ import torch
 import torch.nn.functional as F
 
 
+MEAN: typing.List[float] = [0.0, 0.0, 0.0]
+STD:  typing.List[float] = [1.0, 1.0, 1.0]
+
+
+def init(mean: typing.List[float], std: typing.List[float]):
+    global MEAN, STD
+    MEAN = mean
+    STD = std
+
+
 class Distortioner(torch.nn.Module, metaclass=abc.ABCMeta):
     def __init__(self):
         super().__init__()
+        self.mean = torch.nn.Parameter(torch.tensor(MEAN)[None, :, None, None], requires_grad=False)
+        self.std =  torch.nn.Parameter(torch.tensor(STD)[None, :, None, None], requires_grad=False)
 
     def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
-        raise NotImplementedError()
+        return self._normalize(self.distort(self._unnormalize(i_co), self._unnormalize(i_en)))
+
+    def _normalize(self, x: torch.Tensor) -> torch.Tensor:
+        return (x - self.mean) / self.std
+
+    def _unnormalize(self, x: torch.Tensor) -> torch.Tensor:
+        return x * self.std + self.mean
+
+
+class Combined(Distortioner):
+    def __init__(self, ps: typing.List[float], w: int, sigmas: typing.List[float]):
+        super().__init__()
+        self.distorioner = [
+            Identity(),
+            *[Dropout(p) for p in ps],
+            *[Cropout(p) for p in ps],
+            *[Crop(p) for p in ps],
+            *[GaussianBlur(w, s) for s in sigmas],
+            JPEGDrop(),
+            JPEGMask(),
+        ]
+    
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+        d = random.choice(self.distorioner)
+        return d.distort(i_co, i_en)
 
 
 class Identity(Distortioner):
-    def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+    def __init__(self):
+        super().__init__()
+
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
         return i_en
 
 
@@ -39,7 +78,7 @@ class Dropout(Distortioner):
         super().__init__()
         self.p = p
 
-    def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
         if self.p == 1: return i_en
         mask = self._create_mask(i_co.shape)
         mask = mask.to(i_co.device)
@@ -55,7 +94,7 @@ class Cropout(Distortioner):
         super().__init__()
         self.p = p
 
-    def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
         if self.p == 1: return i_en
         mask = self._create_mask(i_co.shape)
         mask = mask.to(i_co.device)
@@ -83,7 +122,7 @@ class Crop(Distortioner):
         super().__init__()
         self.p = p
 
-    def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
         if self.p == 1: return i_en
         h, w, cy, cx = self._sample_params(i_en.shape)
         return i_en[..., cy-h//2 : cy+h//2, cx-w//2 : cx+w//2]
@@ -108,18 +147,18 @@ class Resize(Distortioner):
         super().__init__()
         self.p = p
 
-    def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
         if self.p == 1: return i_en
         return F.interpolate(i_en, scale_factor=self.p, recompute_scale_factor=True)
 
 
 class GaussianBlur(Distortioner):
-    def __init__(self, w, s: int):
+    def __init__(self, w: int, s: float):
         super().__init__()
         self.w = w
         self.s = s
     
-    def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
         if self.s == 0: return i_en
         return kornia.filters.gaussian_blur2d(i_en, (self.w, self.w), (self.s, self.s))
 
@@ -130,7 +169,7 @@ class JPEGBase(Distortioner):
         self.dct_kernel = torch.nn.Parameter(self._create_dct_basis_matrix(), requires_grad=False)
         self.idct_kernel = torch.nn.Parameter(self._create_idct_basis_matrix(), requires_grad=False)
 
-    def forward(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
+    def distort(self, i_co, i_en: torch.FloatTensor) -> torch.FloatTensor:
         _, _, h, w = i_en.shape
         if not (h % 8 == 0 and w % 8 == 0): raise TypeError("x shape cannot be devided 8")
         x = kornia.color.rgb_to_yuv(i_en) * 255
@@ -199,7 +238,7 @@ class JPEGBase(Distortioner):
 
 
 class JPEGCompression(JPEGBase):
-    def __init__(self, qf=50):
+    def __init__(self, qf: int):
         super().__init__()
         self.qf = qf
         self.kernel = torch.nn.Parameter(self._create_quantization_table(), requires_grad=False)
@@ -222,28 +261,15 @@ class JPEGCompression(JPEGBase):
         return torch.where(qt == 0, torch.ones_like(qt), qt)
 
 
-class JPEGDifferential(JPEGBase):
-    def __init__(self, mean: typing.List[float], std: typing.List[float]):
+class JPEGMask(JPEGBase):
+    def __init__(self):
         super().__init__()
-        self.mean = torch.tensor(mean)[None, :, None, None]
-        self.std = torch.tensor(std)[None, :, None, None]
+        self.kernel = torch.nn.Parameter(self._create_mask_kernel(), requires_grad=False)
     
     def compress(self, x: torch.FloatTensor) -> torch.FloatTensor:
         _, _, h, w = x.shape
-        return self._normalize(self._unnormalize(x) * self.kernel.repeat(1, h//8, w//8))
+        return x * self.kernel.repeat(1, h//8, w//8)
 
-    def _normalize(self, x: torch.Tensor) -> torch.Tensor:
-        return (x - self.mean) / self.std
-
-    def _unnormalize(self, x: torch.Tensor) -> torch.Tensor:
-        return x * self.std + self.mean
-
-
-class JPEGMask(JPEGDifferential):
-    def __init__(self, mean: typing.List[float], std: typing.List[float]):
-        super().__init__(mean, std)
-        self.kernel = torch.nn.Parameter(self._create_mask_kernel(), requires_grad=False)
-    
     def _create_mask_kernel(self):
         kernel = torch.zeros(3, 8, 8)
         kernel[0, :5, :5] = 1 # y
@@ -251,11 +277,15 @@ class JPEGMask(JPEGDifferential):
         return kernel
 
 
-class JPEGDrop(JPEGDifferential):
-    def __init__(self, mean: typing.List[float], std: typing.List[float]):
-        super().__init__(mean, std)
-        self.kernel = torch.nn.Parameter(self._create_drop_kernel(), requires_grad=False)
+class JPEGDrop(JPEGBase):
+    def __init__(self):
+        super().__init__()
     
+    def compress(self, x: torch.FloatTensor) -> torch.FloatTensor:
+        _, _, h, w = x.shape
+        kernel = self._create_drop_kernel().to(x.device)
+        return x * kernel.repeat(1, h//8, w//8)
+
     def _create_drop_kernel(self):
         qt_y = torch.tensor(quantization_table_y, dtype=torch.float64).view(8, 8)
         qt_uv = torch.tensor(quantization_table_uv, dtype=torch.float64).view(8, 8)
@@ -264,7 +294,7 @@ class JPEGDrop(JPEGDifferential):
 
         one = torch.ones(8, 8)
         zero = torch.zeros(8, 8)
-        kernel_y = torch.where(torch.rand(8, 8) > prob_y, one, zero)
+        kernel_y = torch.where(torch.rand(8, 8) > prob_y,  one, zero)
         kernel_u = torch.where(torch.rand(8, 8) > prob_uv, one, zero)
         kernel_v = torch.where(torch.rand(8, 8) > prob_uv, one, zero)
 
